### ms-comev.idl
"""
RPC definitions for the following interfaces:
- IEventObjectCollection (v0.0): f89ac270-d4eb-11d1-b682-00805fc79216
- IEventSystem (v0.0): 4E14FB9F-2E22-11D1-9964-00C04FBBB345
- IEventClass (v0.0): fb2b72a0-7a68-11d1-88f9-0080c7d771bf
- IEventClass2 (v0.0): fb2b72a1-7a68-11d1-88f9-0080c7d771bf
- IEnumEventObject (v0.0): F4A07D63-2E25-11D1-9964-00C04FBBB345
- IEventSubscription (v0.0): 4A6B0E15-2E38-11D1-9965-00C04FBBB345
- IEventSubscription2 (v0.0): 4A6B0E16-2E38-11D1-9965-00C04FBBB345
- IEventClass3 (v0.0): 7FB7EA43-2D76-4ea8-8CD9-3DECC270295E
- IEventSubscription3 (v0.0): FBC1D17D-C498-43a0-81AF-423DDD530AF6
- IEventSystem2 (v0.0): 99CC098F-A48A-4e9c-8E58-965C0AFC19D5
- IEventSystemInitialize (v0.0): a0e8f27a-888c-11d1-b763-00c04fb926af
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import PacketListField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarPacketListField,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRRecursiveField,
    NDRRefEmbPointerField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    register_com_interface,
    register_dcerpc_interface,
)


class MInterfacePointer(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["abData"]
    fields_desc = [
        NDRIntField("ulCntData", None, size_of="abData"),
        NDRConfStrLenField(
            "abData", "", size_is=lambda pkt: pkt.ulCntData, conformant_in_struct=True
        ),
    ]


class get__NewEnum_Request(NDRPacket):
    fields_desc = []


class get__NewEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppUnkEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_R4 = 4
    VT_R8 = 5
    VT_CY = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH = 9
    VT_ERROR = 10
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_UNKNOWN = 13
    VT_DECIMAL = 14
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_INT = 22
    VT_UINT = 23
    VT_VOID = 24
    VT_HRESULT = 25
    VT_PTR = 26
    VT_SAFEARRAY = 27
    VT_CARRAY = 28
    VT_USERDEFINED = 29
    VT_LPSTR = 30
    VT_LPWSTR = 31
    VT_RECORD = 36
    VT_INT_PTR = 37
    VT_UINT_PTR = 38
    VT_ARRAY = 8192
    VT_BYREF = 16384


class CURRENCY(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("int64", 0)]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_R4 = 4
    VT_R8 = 5
    VT_CY = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH = 9
    VT_ERROR = 10
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_UNKNOWN = 13
    VT_DECIMAL = 14
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_INT = 22
    VT_UINT = 23
    VT_VOID = 24
    VT_HRESULT = 25
    VT_PTR = 26
    VT_SAFEARRAY = 27
    VT_CARRAY = 28
    VT_USERDEFINED = 29
    VT_LPSTR = 30
    VT_LPWSTR = 31
    VT_RECORD = 36
    VT_INT_PTR = 37
    VT_UINT_PTR = 38
    VT_ARRAY = 8192
    VT_BYREF = 16384


class SF_TYPE(IntEnum):
    SF_ERROR = VARENUM.VT_ERROR
    SF_I1 = VARENUM.VT_I1
    SF_I2 = VARENUM.VT_I2
    SF_I4 = VARENUM.VT_I4
    SF_I8 = VARENUM.VT_I8
    SF_BSTR = VARENUM.VT_BSTR
    SF_UNKNOWN = VARENUM.VT_UNKNOWN
    SF_DISPATCH = VARENUM.VT_DISPATCH
    SF_VARIANT = VARENUM.VT_VARIANT
    SF_RECORD = VARENUM.VT_RECORD
    SF_HAVEIID = VARENUM.VT_UNKNOWN | 32768


class SAFEARR_BSTR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="aBstr"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "aBstr", [], FLAGGED_WORD_BLOB, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class SAFEARR_UNKNOWN(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apUnknown"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apUnknown", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class SAFEARR_DISPATCH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apDispatch"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apDispatch", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class wireVARIANTStr(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("clSize", 0),
        NDRIntField("rpcReserved", 0),
        NDRShortField("vt", 0),
        NDRShortField("wReserved1", 0),
        NDRShortField("wReserved2", 0),
        NDRShortField("wReserved3", 0),
        NDRRecursiveField("_varUnion"),
    ]


class get_Item_Request(NDRPacket):
    fields_desc = [NDRPacketField("objectID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)]


class get_Item_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("pItem", wireVARIANTStr(), wireVARIANTStr)),
        NDRIntField("status", 0),
    ]


class get_NewEnum_Request(NDRPacket):
    fields_desc = []


class get_NewEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class get_Count_Request(NDRPacket):
    fields_desc = []


class get_Count_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pCount", 0), NDRIntField("status", 0)]


class Add_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("item", wireVARIANTStr(), wireVARIANTStr)),
        NDRPacketField("objectID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class Add_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Remove_Request(NDRPacket):
    fields_desc = [NDRPacketField("objectID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)]


class Remove_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTOBJECTCOLLECTION_OPNUMS = {
    0: DceRpcOp(get__NewEnum_Request, get__NewEnum_Response),
    1: DceRpcOp(get_Item_Request, get_Item_Response),
    2: DceRpcOp(get_NewEnum_Request, get_NewEnum_Response),
    3: DceRpcOp(get_Count_Request, get_Count_Response),
    4: DceRpcOp(Add_Request, Add_Response),
    5: DceRpcOp(Remove_Request, Remove_Response),
}
register_com_interface(
    name="IEventObjectCollection",
    uuid=uuid.UUID("f89ac270-d4eb-11d1-b682-00805fc79216"),
    opnums=IEVENTOBJECTCOLLECTION_OPNUMS,
)


class Query_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("progID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("queryCriteria", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class Query_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("errorIndex", 0),
        NDRFullPointerField(
            NDRPacketField("ppInterface", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class Store_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ProgID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("pInterface", MInterfacePointer(), MInterfacePointer),
    ]


class Store_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Remove_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("progID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("queryCriteria", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class Remove_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("errorIndex", 0), NDRIntField("status", 0)]


class get_EventObjectChangeEventClassID_Request(NDRPacket):
    fields_desc = []


class get_EventObjectChangeEventClassID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrEventClassID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class QueryS_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("progID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("queryCriteria", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class QueryS_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppInterface", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class RemoveS_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("progID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("queryCriteria", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class RemoveS_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTSYSTEM_OPNUMS = {
    0: DceRpcOp(Query_Request, Query_Response),
    1: DceRpcOp(Store_Request, Store_Response),
    2: DceRpcOp(Remove_Request, Remove_Response),
    3: DceRpcOp(
        get_EventObjectChangeEventClassID_Request,
        get_EventObjectChangeEventClassID_Response,
    ),
    4: DceRpcOp(QueryS_Request, QueryS_Response),
    5: DceRpcOp(RemoveS_Request, RemoveS_Response),
}
register_com_interface(
    name="IEventSystem",
    uuid=uuid.UUID("4E14FB9F-2E22-11D1-9964-00C04FBBB345"),
    opnums=IEVENTSYSTEM_OPNUMS,
)


class get_EventClassID_Request(NDRPacket):
    fields_desc = []


class get_EventClassID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrEventClassID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_EventClassID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrEventClassID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_EventClassID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_EventClassName_Request(NDRPacket):
    fields_desc = []


class get_EventClassName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrEventClassName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_EventClassName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrEventClassName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_EventClassName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_OwnerSID_Request(NDRPacket):
    fields_desc = []


class get_OwnerSID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrOwnerSID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_OwnerSID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrOwnerSID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_OwnerSID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_FiringInterfaceID_Request(NDRPacket):
    fields_desc = []


class get_FiringInterfaceID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrFiringInterfaceID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_FiringInterfaceID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrFiringInterfaceID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_FiringInterfaceID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_Description_Request(NDRPacket):
    fields_desc = []


class get_Description_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrDescription", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_Description_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrDescription", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_Description_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_TypeLib_Request(NDRPacket):
    fields_desc = []


class get_TypeLib_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrTypeLib", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_TypeLib_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrTypeLib", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_TypeLib_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTCLASS_OPNUMS = {
    0: DceRpcOp(get_EventClassID_Request, get_EventClassID_Response),
    1: DceRpcOp(put_EventClassID_Request, put_EventClassID_Response),
    2: DceRpcOp(get_EventClassName_Request, get_EventClassName_Response),
    3: DceRpcOp(put_EventClassName_Request, put_EventClassName_Response),
    4: DceRpcOp(get_OwnerSID_Request, get_OwnerSID_Response),
    5: DceRpcOp(put_OwnerSID_Request, put_OwnerSID_Response),
    6: DceRpcOp(get_FiringInterfaceID_Request, get_FiringInterfaceID_Response),
    7: DceRpcOp(put_FiringInterfaceID_Request, put_FiringInterfaceID_Response),
    8: DceRpcOp(get_Description_Request, get_Description_Response),
    9: DceRpcOp(put_Description_Request, put_Description_Response),
    # 10: Opnum17NotUsedOnWire,
    # 11: Opnum18NotUsedOnWire,
    12: DceRpcOp(get_TypeLib_Request, get_TypeLib_Response),
    13: DceRpcOp(put_TypeLib_Request, put_TypeLib_Response),
}
register_com_interface(
    name="IEventClass",
    uuid=uuid.UUID("fb2b72a0-7a68-11d1-88f9-0080c7d771bf"),
    opnums=IEVENTCLASS_OPNUMS,
)


class get_PublisherID_Request(NDRPacket):
    fields_desc = []


class get_PublisherID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrPublisherID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_PublisherID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPublisherID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_PublisherID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_MultiInterfacePublisherFilterCLSID_Request(NDRPacket):
    fields_desc = []


class get_MultiInterfacePublisherFilterCLSID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrPubFilCLSID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_MultiInterfacePublisherFilterCLSID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPubFilCLSID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_MultiInterfacePublisherFilterCLSID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_AllowInprocActivation_Request(NDRPacket):
    fields_desc = []


class get_AllowInprocActivation_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("pfAllowInprocActivation", 0),
        NDRIntField("status", 0),
    ]


class put_AllowInprocActivation_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("fAllowInprocActivation", 0)]


class put_AllowInprocActivation_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_FireInParallel_Request(NDRPacket):
    fields_desc = []


class get_FireInParallel_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pfFireInParallel", 0), NDRIntField("status", 0)]


class put_FireInParallel_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("fFireInParallel", 0)]


class put_FireInParallel_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTCLASS2_OPNUMS = {
    0: DceRpcOp(get_PublisherID_Request, get_PublisherID_Response),
    1: DceRpcOp(put_PublisherID_Request, put_PublisherID_Response),
    2: DceRpcOp(
        get_MultiInterfacePublisherFilterCLSID_Request,
        get_MultiInterfacePublisherFilterCLSID_Response,
    ),
    3: DceRpcOp(
        put_MultiInterfacePublisherFilterCLSID_Request,
        put_MultiInterfacePublisherFilterCLSID_Response,
    ),
    4: DceRpcOp(get_AllowInprocActivation_Request, get_AllowInprocActivation_Response),
    5: DceRpcOp(put_AllowInprocActivation_Request, put_AllowInprocActivation_Response),
    6: DceRpcOp(get_FireInParallel_Request, get_FireInParallel_Response),
    7: DceRpcOp(put_FireInParallel_Request, put_FireInParallel_Response),
}
register_com_interface(
    name="IEventClass2",
    uuid=uuid.UUID("fb2b72a1-7a68-11d1-88f9-0080c7d771bf"),
    opnums=IEVENTCLASS2_OPNUMS,
)


class Clone_Request(NDRPacket):
    fields_desc = []


class Clone_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppInterface", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class Next_Request(NDRPacket):
    fields_desc = [NDRIntField("cReqElem", 0)]


class Next_Response(NDRPacket):
    fields_desc = [
        NDRConfVarPacketListField(
            "ppInterface",
            [],
            MInterfacePointer,
            size_is=lambda pkt: pkt.cReqElem,
            length_is=lambda pkt: pkt.cRetElem,
            ptr_pack=True,
        ),
        NDRIntField("cRetElem", None, size_of="ppInterface"),
        NDRIntField("status", 0),
    ]


class Reset_Request(NDRPacket):
    fields_desc = []


class Reset_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Skip_Request(NDRPacket):
    fields_desc = [NDRIntField("cSkipElem", 0)]


class Skip_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IENUMEVENTOBJECT_OPNUMS = {
    0: DceRpcOp(Clone_Request, Clone_Response),
    1: DceRpcOp(Next_Request, Next_Response),
    2: DceRpcOp(Reset_Request, Reset_Response),
    3: DceRpcOp(Skip_Request, Skip_Response),
}
register_com_interface(
    name="IEnumEventObject",
    uuid=uuid.UUID("F4A07D63-2E25-11D1-9964-00C04FBBB345"),
    opnums=IENUMEVENTOBJECT_OPNUMS,
)


class get_SubscriptionID_Request(NDRPacket):
    fields_desc = []


class get_SubscriptionID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrSubscriptionID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_SubscriptionID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrSubscriptionID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_SubscriptionID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_SubscriptionName_Request(NDRPacket):
    fields_desc = []


class get_SubscriptionName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrSubscriptionName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_SubscriptionName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrSubscriptionName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_SubscriptionName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_PublisherID_Request(NDRPacket):
    fields_desc = []


class get_PublisherID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrPublisherID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_PublisherID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPublisherID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_PublisherID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_EventClassID_Request(NDRPacket):
    fields_desc = []


class get_EventClassID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrEventClassID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_EventClassID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrEventClassID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_EventClassID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_MethodName_Request(NDRPacket):
    fields_desc = []


class get_MethodName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrMethodName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_MethodName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrMethodName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_MethodName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_SubscriberCLSID_Request(NDRPacket):
    fields_desc = []


class get_SubscriberCLSID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrSubscriberCLSID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_SubscriberCLSID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrSubscriberCLSID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_SubscriberCLSID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_SubscriberInterface_Request(NDRPacket):
    fields_desc = []


class get_SubscriberInterface_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ppSubscriberInterface", MInterfacePointer(), MInterfacePointer
            )
        ),
        NDRIntField("status", 0),
    ]


class put_SubscriberInterface_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pSubscriberInterface", MInterfacePointer(), MInterfacePointer)
    ]


class put_SubscriberInterface_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_PerUser_Request(NDRPacket):
    fields_desc = []


class get_PerUser_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pfPerUser", 0), NDRIntField("status", 0)]


class put_PerUser_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("fPerUser", 0)]


class put_PerUser_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_OwnerSID_Request(NDRPacket):
    fields_desc = []


class get_OwnerSID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrOwnerSID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_OwnerSID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrOwnerSID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_OwnerSID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_Enabled_Request(NDRPacket):
    fields_desc = []


class get_Enabled_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pfEnabled", 0), NDRIntField("status", 0)]


class put_Enabled_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("fEnabled", 0)]


class put_Enabled_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_Description_Request(NDRPacket):
    fields_desc = []


class get_Description_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrDescription", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_Description_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrDescription", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_Description_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_MachineName_Request(NDRPacket):
    fields_desc = []


class get_MachineName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrMachineName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_MachineName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrMachineName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_MachineName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetPublisherProperty_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPropertyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetPublisherProperty_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("propertyValue", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class PutPublisherProperty_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPropertyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(
            NDRPacketField("propertyValue", wireVARIANTStr(), wireVARIANTStr)
        ),
    ]


class PutPublisherProperty_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RemovePublisherProperty_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPropertyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class RemovePublisherProperty_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetPublisherPropertyCollection_Request(NDRPacket):
    fields_desc = []


class GetPublisherPropertyCollection_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("collection", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class GetSubscriberProperty_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPropertyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetSubscriberProperty_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("propertyValue", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class PutSubscriberProperty_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPropertyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(
            NDRPacketField("propertyValue", wireVARIANTStr(), wireVARIANTStr)
        ),
    ]


class PutSubscriberProperty_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RemoveSubscriberProperty_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPropertyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class RemoveSubscriberProperty_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetSubscriberPropertyCollection_Request(NDRPacket):
    fields_desc = []


class GetSubscriberPropertyCollection_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("collection", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class get_InterfaceID_Request(NDRPacket):
    fields_desc = []


class get_InterfaceID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrInterfaceID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_InterfaceID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrInterfaceID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_InterfaceID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTSUBSCRIPTION_OPNUMS = {
    0: DceRpcOp(get_SubscriptionID_Request, get_SubscriptionID_Response),
    1: DceRpcOp(put_SubscriptionID_Request, put_SubscriptionID_Response),
    2: DceRpcOp(get_SubscriptionName_Request, get_SubscriptionName_Response),
    3: DceRpcOp(put_SubscriptionName_Request, put_SubscriptionName_Response),
    4: DceRpcOp(get_PublisherID_Request, get_PublisherID_Response),
    5: DceRpcOp(put_PublisherID_Request, put_PublisherID_Response),
    6: DceRpcOp(get_EventClassID_Request, get_EventClassID_Response),
    7: DceRpcOp(put_EventClassID_Request, put_EventClassID_Response),
    8: DceRpcOp(get_MethodName_Request, get_MethodName_Response),
    9: DceRpcOp(put_MethodName_Request, put_MethodName_Response),
    10: DceRpcOp(get_SubscriberCLSID_Request, get_SubscriberCLSID_Response),
    11: DceRpcOp(put_SubscriberCLSID_Request, put_SubscriberCLSID_Response),
    12: DceRpcOp(get_SubscriberInterface_Request, get_SubscriberInterface_Response),
    13: DceRpcOp(put_SubscriberInterface_Request, put_SubscriberInterface_Response),
    14: DceRpcOp(get_PerUser_Request, get_PerUser_Response),
    15: DceRpcOp(put_PerUser_Request, put_PerUser_Response),
    16: DceRpcOp(get_OwnerSID_Request, get_OwnerSID_Response),
    17: DceRpcOp(put_OwnerSID_Request, put_OwnerSID_Response),
    18: DceRpcOp(get_Enabled_Request, get_Enabled_Response),
    19: DceRpcOp(put_Enabled_Request, put_Enabled_Response),
    20: DceRpcOp(get_Description_Request, get_Description_Response),
    21: DceRpcOp(put_Description_Request, put_Description_Response),
    22: DceRpcOp(get_MachineName_Request, get_MachineName_Response),
    23: DceRpcOp(put_MachineName_Request, put_MachineName_Response),
    24: DceRpcOp(GetPublisherProperty_Request, GetPublisherProperty_Response),
    25: DceRpcOp(PutPublisherProperty_Request, PutPublisherProperty_Response),
    26: DceRpcOp(RemovePublisherProperty_Request, RemovePublisherProperty_Response),
    27: DceRpcOp(
        GetPublisherPropertyCollection_Request, GetPublisherPropertyCollection_Response
    ),
    28: DceRpcOp(GetSubscriberProperty_Request, GetSubscriberProperty_Response),
    29: DceRpcOp(PutSubscriberProperty_Request, PutSubscriberProperty_Response),
    30: DceRpcOp(RemoveSubscriberProperty_Request, RemoveSubscriberProperty_Response),
    31: DceRpcOp(
        GetSubscriberPropertyCollection_Request,
        GetSubscriberPropertyCollection_Response,
    ),
    32: DceRpcOp(get_InterfaceID_Request, get_InterfaceID_Response),
    33: DceRpcOp(put_InterfaceID_Request, put_InterfaceID_Response),
}
register_com_interface(
    name="IEventSubscription",
    uuid=uuid.UUID("4A6B0E15-2E38-11D1-9965-00C04FBBB345"),
    opnums=IEVENTSUBSCRIPTION_OPNUMS,
)


class get_FilterCriteria_Request(NDRPacket):
    fields_desc = []


class get_FilterCriteria_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrFilterCriteria", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_FilterCriteria_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrFilterCriteria", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_FilterCriteria_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_SubscriberMoniker_Request(NDRPacket):
    fields_desc = []


class get_SubscriberMoniker_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrMoniker", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class put_SubscriberMoniker_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrMoniker", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class put_SubscriberMoniker_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTSUBSCRIPTION2_OPNUMS = {
    0: DceRpcOp(get_FilterCriteria_Request, get_FilterCriteria_Response),
    1: DceRpcOp(put_FilterCriteria_Request, put_FilterCriteria_Response),
    2: DceRpcOp(get_SubscriberMoniker_Request, get_SubscriberMoniker_Response),
    3: DceRpcOp(put_SubscriberMoniker_Request, put_SubscriberMoniker_Response),
}
register_com_interface(
    name="IEventSubscription2",
    uuid=uuid.UUID("4A6B0E16-2E38-11D1-9965-00C04FBBB345"),
    opnums=IEVENTSUBSCRIPTION2_OPNUMS,
)


class get_EventClassPartitionID_Request(NDRPacket):
    fields_desc = []


class get_EventClassPartitionID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrEventClassPartitionID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_EventClassPartitionID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrEventClassPartitionID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        )
    ]


class put_EventClassPartitionID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_EventClassApplicationID_Request(NDRPacket):
    fields_desc = []


class get_EventClassApplicationID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrEventClassApplicationID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_EventClassApplicationID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrEventClassApplicationID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        )
    ]


class put_EventClassApplicationID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTCLASS3_OPNUMS = {
    0: DceRpcOp(get_EventClassPartitionID_Request, get_EventClassPartitionID_Response),
    1: DceRpcOp(put_EventClassPartitionID_Request, put_EventClassPartitionID_Response),
    2: DceRpcOp(
        get_EventClassApplicationID_Request, get_EventClassApplicationID_Response
    ),
    3: DceRpcOp(
        put_EventClassApplicationID_Request, put_EventClassApplicationID_Response
    ),
}
register_com_interface(
    name="IEventClass3",
    uuid=uuid.UUID("7FB7EA43-2D76-4ea8-8CD9-3DECC270295E"),
    opnums=IEVENTCLASS3_OPNUMS,
)


class get_EventClassPartitionID_Request(NDRPacket):
    fields_desc = []


class get_EventClassPartitionID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrEventClassPartitionID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_EventClassPartitionID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrEventClassPartitionID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        )
    ]


class put_EventClassPartitionID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_EventClassApplicationID_Request(NDRPacket):
    fields_desc = []


class get_EventClassApplicationID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrEventClassApplicationID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_EventClassApplicationID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrEventClassApplicationID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        )
    ]


class put_EventClassApplicationID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_SubscriberPartitionID_Request(NDRPacket):
    fields_desc = []


class get_SubscriberPartitionID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrSubscriberPartitionID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_SubscriberPartitionID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrSubscriberPartitionID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        )
    ]


class put_SubscriberPartitionID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class get_SubscriberApplicationID_Request(NDRPacket):
    fields_desc = []


class get_SubscriberApplicationID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrSubscriberApplicationID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class put_SubscriberApplicationID_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrSubscriberApplicationID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        )
    ]


class put_SubscriberApplicationID_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTSUBSCRIPTION3_OPNUMS = {
    0: DceRpcOp(get_EventClassPartitionID_Request, get_EventClassPartitionID_Response),
    1: DceRpcOp(put_EventClassPartitionID_Request, put_EventClassPartitionID_Response),
    2: DceRpcOp(
        get_EventClassApplicationID_Request, get_EventClassApplicationID_Response
    ),
    3: DceRpcOp(
        put_EventClassApplicationID_Request, put_EventClassApplicationID_Response
    ),
    4: DceRpcOp(get_SubscriberPartitionID_Request, get_SubscriberPartitionID_Response),
    5: DceRpcOp(put_SubscriberPartitionID_Request, put_SubscriberPartitionID_Response),
    6: DceRpcOp(
        get_SubscriberApplicationID_Request, get_SubscriberApplicationID_Response
    ),
    7: DceRpcOp(
        put_SubscriberApplicationID_Request, put_SubscriberApplicationID_Response
    ),
}
register_com_interface(
    name="IEventSubscription3",
    uuid=uuid.UUID("FBC1D17D-C498-43a0-81AF-423DDD530AF6"),
    opnums=IEVENTSUBSCRIPTION3_OPNUMS,
)


class GetVersion_Request(NDRPacket):
    fields_desc = []


class GetVersion_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pnVersion", 0), NDRIntField("status", 0)]


class VerifyTransientSubscribers_Request(NDRPacket):
    fields_desc = []


class VerifyTransientSubscribers_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTSYSTEM2_OPNUMS = {
    0: DceRpcOp(GetVersion_Request, GetVersion_Response),
    1: DceRpcOp(
        VerifyTransientSubscribers_Request, VerifyTransientSubscribers_Response
    ),
}
register_com_interface(
    name="IEventSystem2",
    uuid=uuid.UUID("99CC098F-A48A-4e9c-8E58-965C0AFC19D5"),
    opnums=IEVENTSYSTEM2_OPNUMS,
)


class SetCOMCatalogBehaviour_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("bRetainSubKeys", 0)]


class SetCOMCatalogBehaviour_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEVENTSYSTEMINITIALIZE_OPNUMS = {
    0: DceRpcOp(SetCOMCatalogBehaviour_Request, SetCOMCatalogBehaviour_Response)
}
register_dcerpc_interface(
    name="IEventSystemInitialize",
    uuid=uuid.UUID("a0e8f27a-888c-11d1-b763-00c04fb926af"),
    version="0.0",
    opnums=IEVENTSYSTEMINITIALIZE_OPNUMS,
)



### ms-dssp.idl
"""
RPC definitions for the following interfaces:
- dssetup (v0.0): 3919286a-b10c-11d0-9ba8-00c04fd92ef5
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRUnionField,
    register_dcerpc_interface,
)


class DSROLE_PRIMARY_DOMAIN_INFO_LEVEL(IntEnum):
    DsRolePrimaryDomainInfoBasic = 1
    DsRoleUpgradeStatus = 2
    DsRoleOperationState = 3


class DSROLE_MACHINE_ROLE(IntEnum):
    DsRole_RoleStandaloneWorkstation = 1
    DsRole_RoleMemberWorkstation = 2
    DsRole_RoleStandaloneServer = 3
    DsRole_RoleMemberServer = 4
    DsRole_RoleBackupDomainController = 5
    DsRole_RolePrimaryDomainController = 6


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class DSROLER_PRIMARY_DOMAIN_INFO_BASIC(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRInt3264EnumField("MachineRole", 0, DSROLE_MACHINE_ROLE),
        NDRIntField("Flags", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("DomainNameFlat", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("DomainNameDns", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("DomainForestName", ""), deferred=True
        ),
        NDRPacketField("DomainGuid", GUID(), GUID),
    ]


class DSROLE_SERVER_STATE(IntEnum):
    DsRoleServerUnknown = 0
    DsRoleServerPrimary = 1
    DsRoleServerBackup = 2


class DSROLE_UPGRADE_STATUS_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("OperationState", 0),
        NDRInt3264EnumField("PreviousServerState", 0, DSROLE_SERVER_STATE),
    ]


class DSROLE_OPERATION_STATE(IntEnum):
    DsRoleOperationIdle = 0
    DsRoleOperationActive = 1
    DsRoleOperationNeedReboot = 2


class DSROLE_OPERATION_STATE_INFO(NDRPacket):
    ALIGNMENT = (2, 4)
    fields_desc = [NDRInt3264EnumField("OperationState", 0, DSROLE_OPERATION_STATE)]


class DsRolerGetPrimaryDomainInformation_Request(NDRPacket):
    fields_desc = [
        NDRInt3264EnumField("InfoLevel", 0, DSROLE_PRIMARY_DOMAIN_INFO_LEVEL)
    ]


class DsRolerGetPrimaryDomainInformation_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRUnionField(
                [
                    (
                        NDRPacketField(
                            "DomainInfo",
                            DSROLER_PRIMARY_DOMAIN_INFO_BASIC(),
                            DSROLER_PRIMARY_DOMAIN_INFO_BASIC,
                        ),
                        (
                            (
                                lambda pkt: getattr(pkt, "InfoLevel", None)
                                == DSROLE_PRIMARY_DOMAIN_INFO_LEVEL.DsRolePrimaryDomainInfoBasic
                            ),
                            (
                                lambda _, val: val.tag
                                == DSROLE_PRIMARY_DOMAIN_INFO_LEVEL.DsRolePrimaryDomainInfoBasic
                            ),
                        ),
                    ),
                    (
                        NDRPacketField(
                            "DomainInfo",
                            DSROLE_UPGRADE_STATUS_INFO(),
                            DSROLE_UPGRADE_STATUS_INFO,
                        ),
                        (
                            (
                                lambda pkt: getattr(pkt, "InfoLevel", None)
                                == DSROLE_PRIMARY_DOMAIN_INFO_LEVEL.DsRoleUpgradeStatus
                            ),
                            (
                                lambda _, val: val.tag
                                == DSROLE_PRIMARY_DOMAIN_INFO_LEVEL.DsRoleUpgradeStatus
                            ),
                        ),
                    ),
                    (
                        NDRPacketField(
                            "DomainInfo",
                            DSROLE_OPERATION_STATE_INFO(),
                            DSROLE_OPERATION_STATE_INFO,
                        ),
                        (
                            (
                                lambda pkt: getattr(pkt, "InfoLevel", None)
                                == DSROLE_PRIMARY_DOMAIN_INFO_LEVEL.DsRoleOperationState
                            ),
                            (
                                lambda _, val: val.tag
                                == DSROLE_PRIMARY_DOMAIN_INFO_LEVEL.DsRoleOperationState
                            ),
                        ),
                    ),
                ],
                StrFixedLenField("DomainInfo", "", length=0),
                align=(2, 8),
                switch_fmt=("H", "I"),
            )
        ),
        NDRIntField("status", 0),
    ]


DSSETUP_OPNUMS = {
    0: DceRpcOp(
        DsRolerGetPrimaryDomainInformation_Request,
        DsRolerGetPrimaryDomainInformation_Response,
    ),
    # 1: Opnum1NotUsedOnWire,
    # 2: Opnum2NotUsedOnWire,
    # 3: Opnum3NotUsedOnWire,
    # 4: Opnum4NotUsedOnWire,
    # 5: Opnum5NotUsedOnWire,
    # 6: Opnum6NotUsedOnWire,
    # 7: Opnum7NotUsedOnWire,
    # 8: Opnum8NotUsedOnWire,
    # 9: Opnum9NotUsedOnWire,
    # 10: Opnum10NotUsedOnWire,
    # 11: Opnum11NotUsedOnWire
}
register_dcerpc_interface(
    name="dssetup",
    uuid=uuid.UUID("3919286a-b10c-11d0-9ba8-00c04fd92ef5"),
    version="0.0",
    opnums=DSSETUP_OPNUMS,
)



### ms-dltm.idl
"""
RPC definitions for the following interfaces:
- trksvr (v1.0): 4da1c422-943d-11d1-acae-00c04fc2aa3f
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField, StrFixedLenFieldUtf16
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntEnumField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedShortField,
    NDRUnionField,
    register_dcerpc_interface,
)


class TRKSVR_MESSAGE_TYPE(IntEnum):
    old_SEARCH = 1
    MOVE_NOTIFICATION = 1
    REFRESH = 2
    SYNC_VOLUMES = 3
    DELETE_NOTIFY = 4
    STATISTICS = 5
    SEARCH = 6
    WKS_CONFIG = 7
    WKS_VOLUME_REFRESH = 8


class TRKSVR_MESSAGE_PRIORITY(IntEnum):
    PRI_0 = 0
    PRI_1 = 1
    PRI_2 = 2
    PRI_3 = 3
    PRI_4 = 4
    PRI_5 = 5
    PRI_6 = 6
    PRI_7 = 7
    PRI_8 = 8
    PRI_9 = 9


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class CVolumeId(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRPacketField("_volume", GUID(), GUID)]


class CObjId(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRPacketField("_object", GUID(), GUID)]


class CDomainRelativeObjId(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("_volume", CVolumeId(), CVolumeId),
        NDRPacketField("_object", CObjId(), CObjId),
    ]


class old_TRK_FILE_TRACKING_INFORMATION(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        StrFixedLenFieldUtf16("tszFilePath", "", length=257 * 2),
        NDRPacketField("droidBirth", CDomainRelativeObjId(), CDomainRelativeObjId),
        NDRPacketField("droidLast", CDomainRelativeObjId(), CDomainRelativeObjId),
        NDRSignedIntField("hr", 0),
    ]


class old_TRKSVR_CALL_SEARCH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cSearch", None, size_of="pSearches"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pSearches",
                [],
                old_TRK_FILE_TRACKING_INFORMATION,
                size_is=lambda pkt: pkt.cSearch,
            ),
            deferred=True,
        ),
    ]


class TRKSVR_CALL_MOVE_NOTIFICATION(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cNotifications", None, size_of="rgdroidNew"),
        NDRIntField("cProcessed", 0),
        NDRSignedIntField("seq", 0),
        NDRSignedIntField("fForceSeqNumber", 0),
        NDRFullPointerField(
            NDRPacketField("pvolid", CVolumeId(), CVolumeId), deferred=True
        ),
        NDRFullPointerField(
            NDRConfPacketListField(
                "rgobjidCurrent", [], CObjId, size_is=lambda pkt: pkt.cNotifications
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfPacketListField(
                "rgdroidBirth",
                [],
                CDomainRelativeObjId,
                size_is=lambda pkt: pkt.cNotifications,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfPacketListField(
                "rgdroidNew",
                [],
                CDomainRelativeObjId,
                size_is=lambda pkt: pkt.cNotifications,
            ),
            deferred=True,
        ),
    ]


class TRKSVR_CALL_REFRESH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cSources", None, size_of="adroidBirth"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "adroidBirth",
                [],
                CDomainRelativeObjId,
                size_is=lambda pkt: pkt.cSources,
            ),
            deferred=True,
        ),
        NDRIntField("cVolumes", None, size_of="avolid"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "avolid", [], CVolumeId, size_is=lambda pkt: pkt.cVolumes
            ),
            deferred=True,
        ),
    ]


class TRKSVR_SYNC_TYPE(IntEnum):
    CREATE_VOLUME = 0
    QUERY_VOLUME = 1
    CLAIM_VOLUME = 2
    FIND_VOLUME = 3
    TEST_VOLUME = 4
    DELETE_VOLUME = 5


class CVolumeSecret(NDRPacket):
    fields_desc = [StrFixedLenField("_abSecret", "", length=8)]


class FILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class CMachineId(NDRPacket):
    fields_desc = [StrFixedLenField("_szMachine", "", length=16)]


class TRKSVR_SYNC_VOLUME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRSignedIntField("hr", 0),
        NDRIntEnumField("SyncType", 0, TRKSVR_SYNC_TYPE),
        NDRPacketField("volume", CVolumeId(), CVolumeId),
        NDRPacketField("secret", CVolumeSecret(), CVolumeSecret),
        NDRPacketField("secretOld", CVolumeSecret(), CVolumeSecret),
        NDRSignedIntField("seq", 0),
        NDRPacketField("ftLastRefresh", FILETIME(), FILETIME),
        NDRPacketField("machine", CMachineId(), CMachineId),
    ]


class TRKSVR_CALL_SYNC_VOLUMES(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cVolumes", None, size_of="pVolumes"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pVolumes", [], TRKSVR_SYNC_VOLUME, size_is=lambda pkt: pkt.cVolumes
            ),
            deferred=True,
        ),
    ]


class TRKSVR_CALL_DELETE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cdroidBirth", None, size_of="adroidBirth"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "adroidBirth",
                [],
                CDomainRelativeObjId,
                size_is=lambda pkt: pkt.cdroidBirth,
            ),
            deferred=True,
        ),
        NDRIntField("cVolumes", None, size_of="pVolumes"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pVolumes", [], CVolumeId, size_is=lambda pkt: pkt.cVolumes
            ),
            deferred=True,
        ),
    ]


class Version_TRKSVR_STATISTICS(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("dwMajor", 0),
        NDRIntField("dwMinor", 0),
        NDRIntField("dwBuildNumber", 0),
    ]


class TRKSVR_STATISTICS(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("cSyncVolumeRequests", 0),
        NDRIntField("cSyncVolumeErrors", 0),
        NDRIntField("cSyncVolumeThreads", 0),
        NDRIntField("cCreateVolumeRequests", 0),
        NDRIntField("cCreateVolumeErrors", 0),
        NDRIntField("cClaimVolumeRequests", 0),
        NDRIntField("cClaimVolumeErrors", 0),
        NDRIntField("cQueryVolumeRequests", 0),
        NDRIntField("cQueryVolumeErrors", 0),
        NDRIntField("cFindVolumeRequests", 0),
        NDRIntField("cFindVolumeErrors", 0),
        NDRIntField("cTestVolumeRequests", 0),
        NDRIntField("cTestVolumeErrors", 0),
        NDRIntField("cSearchRequests", 0),
        NDRIntField("cSearchErrors", 0),
        NDRIntField("cSearchThreads", 0),
        NDRIntField("cMoveNotifyRequests", 0),
        NDRIntField("cMoveNotifyErrors", 0),
        NDRIntField("cMoveNotifyThreads", 0),
        NDRIntField("cRefreshRequests", 0),
        NDRIntField("cRefreshErrors", 0),
        NDRIntField("cRefreshThreads", 0),
        NDRIntField("cDeleteNotifyRequests", 0),
        NDRIntField("cDeleteNotifyErrors", 0),
        NDRIntField("cDeleteNotifyThreads", 0),
        NDRIntField("ulGCIterationPeriod", 0),
        NDRPacketField("ftLastSuccessfulRequest", FILETIME(), FILETIME),
        NDRSignedIntField("hrLastError", 0),
        NDRIntField("dwMoveLimit", 0),
        NDRSignedIntField("lRefreshCounter", 0),
        NDRIntField("dwCachedVolumeTableCount", 0),
        NDRIntField("dwCachedMoveTableCount", 0),
        NDRPacketField("ftCachedLastUpdated", FILETIME(), FILETIME),
        NDRSignedIntField("fIsDesignatedDc", 0),
        NDRPacketField("ftNextGC", FILETIME(), FILETIME),
        NDRPacketField("ftServiceStart", FILETIME(), FILETIME),
        NDRIntField("cMaxRPCThreads", 0),
        NDRIntField("cAvailableRPCThreads", 0),
        NDRIntField("cLowestAvailableRPCThreads", 0),
        NDRIntField("cNumThreadPoolThreads", 0),
        NDRIntField("cMostThreadPoolThreads", 0),
        NDRSignedShortField("cEntriesToGC", 0),
        NDRSignedShortField("cEntriesGCed", 0),
        NDRSignedShortField("cMaxDsWriteEvents", 0),
        NDRSignedShortField("cCurrentFailedWrites", 0),
        NDRPacketField(
            "Version_TRKSVR_STATISTICS",
            Version_TRKSVR_STATISTICS(),
            Version_TRKSVR_STATISTICS,
        ),
    ]


class TRK_FILE_TRACKING_INFORMATION(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("droidBirth", CDomainRelativeObjId(), CDomainRelativeObjId),
        NDRPacketField("droidLast", CDomainRelativeObjId(), CDomainRelativeObjId),
        NDRPacketField("mcidLast", CMachineId(), CMachineId),
        NDRSignedIntField("hr", 0),
    ]


class TRKSVR_CALL_SEARCH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cSearch", None, size_of="pSearches"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pSearches",
                [],
                TRK_FILE_TRACKING_INFORMATION,
                size_is=lambda pkt: pkt.cSearch,
            ),
            deferred=True,
        ),
    ]


class TRKWKS_CONFIG(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwParameter", 0), NDRIntField("dwNewValue", 0)]


class TRKSVR_MESSAGE_UNION(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntEnumField("MessageType", 0, TRKSVR_MESSAGE_TYPE),
        NDRIntEnumField("Priority", 0, TRKSVR_MESSAGE_PRIORITY),
        NDRUnionField(
            [
                (
                    NDRPacketField(
                        "value", old_TRKSVR_CALL_SEARCH(), old_TRKSVR_CALL_SEARCH
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "MessageType", None)
                            == TRKSVR_MESSAGE_TYPE.old_SEARCH
                        ),
                        (lambda _, val: val.tag == TRKSVR_MESSAGE_TYPE.old_SEARCH),
                    ),
                ),
                (
                    NDRPacketField(
                        "value",
                        TRKSVR_CALL_MOVE_NOTIFICATION(),
                        TRKSVR_CALL_MOVE_NOTIFICATION,
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "MessageType", None)
                            == TRKSVR_MESSAGE_TYPE.MOVE_NOTIFICATION
                        ),
                        (
                            lambda _, val: val.tag
                            == TRKSVR_MESSAGE_TYPE.MOVE_NOTIFICATION
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", TRKSVR_CALL_REFRESH(), TRKSVR_CALL_REFRESH),
                    (
                        (
                            lambda pkt: getattr(pkt, "MessageType", None)
                            == TRKSVR_MESSAGE_TYPE.REFRESH
                        ),
                        (lambda _, val: val.tag == TRKSVR_MESSAGE_TYPE.REFRESH),
                    ),
                ),
                (
                    NDRPacketField(
                        "value", TRKSVR_CALL_SYNC_VOLUMES(), TRKSVR_CALL_SYNC_VOLUMES
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "MessageType", None)
                            == TRKSVR_MESSAGE_TYPE.SYNC_VOLUMES
                        ),
                        (lambda _, val: val.tag == TRKSVR_MESSAGE_TYPE.SYNC_VOLUMES),
                    ),
                ),
                (
                    NDRPacketField("value", TRKSVR_CALL_DELETE(), TRKSVR_CALL_DELETE),
                    (
                        (
                            lambda pkt: getattr(pkt, "MessageType", None)
                            == TRKSVR_MESSAGE_TYPE.DELETE_NOTIFY
                        ),
                        (lambda _, val: val.tag == TRKSVR_MESSAGE_TYPE.DELETE_NOTIFY),
                    ),
                ),
                (
                    NDRPacketField("value", TRKSVR_STATISTICS(), TRKSVR_STATISTICS),
                    (
                        (
                            lambda pkt: getattr(pkt, "MessageType", None)
                            == TRKSVR_MESSAGE_TYPE.STATISTICS
                        ),
                        (lambda _, val: val.tag == TRKSVR_MESSAGE_TYPE.STATISTICS),
                    ),
                ),
                (
                    NDRPacketField("value", TRKSVR_CALL_SEARCH(), TRKSVR_CALL_SEARCH),
                    (
                        (
                            lambda pkt: getattr(pkt, "MessageType", None)
                            == TRKSVR_MESSAGE_TYPE.SEARCH
                        ),
                        (lambda _, val: val.tag == TRKSVR_MESSAGE_TYPE.SEARCH),
                    ),
                ),
                (
                    NDRPacketField("value", TRKWKS_CONFIG(), TRKWKS_CONFIG),
                    (
                        (
                            lambda pkt: getattr(pkt, "MessageType", None)
                            == TRKSVR_MESSAGE_TYPE.WKS_CONFIG
                        ),
                        (lambda _, val: val.tag == TRKSVR_MESSAGE_TYPE.WKS_CONFIG),
                    ),
                ),
                (
                    NDRIntField("value", 0),
                    (
                        (
                            lambda pkt: getattr(pkt, "MessageType", None)
                            == TRKSVR_MESSAGE_TYPE.WKS_VOLUME_REFRESH
                        ),
                        (
                            lambda _, val: val.tag
                            == TRKSVR_MESSAGE_TYPE.WKS_VOLUME_REFRESH
                        ),
                    ),
                ),
            ],
            StrFixedLenField("value", "", length=0),
            align=(4, 8),
            switch_fmt=("I", "I"),
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ptszMachineID", ""), deferred=True
        ),
    ]


class LnkSvrMessage_Request(NDRPacket):
    fields_desc = [NDRPacketField("pMsg", TRKSVR_MESSAGE_UNION(), TRKSVR_MESSAGE_UNION)]


class LnkSvrMessage_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pMsg", TRKSVR_MESSAGE_UNION(), TRKSVR_MESSAGE_UNION),
        NDRIntField("status", 0),
    ]


class LnkSvrMessageCallback_Request(NDRPacket):
    fields_desc = [NDRPacketField("pMsg", TRKSVR_MESSAGE_UNION(), TRKSVR_MESSAGE_UNION)]


class LnkSvrMessageCallback_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pMsg", TRKSVR_MESSAGE_UNION(), TRKSVR_MESSAGE_UNION),
        NDRIntField("status", 0),
    ]


TRKSVR_OPNUMS = {
    0: DceRpcOp(LnkSvrMessage_Request, LnkSvrMessage_Response),
    1: DceRpcOp(LnkSvrMessageCallback_Request, LnkSvrMessageCallback_Response),
}
register_dcerpc_interface(
    name="trksvr",
    uuid=uuid.UUID("4da1c422-943d-11d1-acae-00c04fc2aa3f"),
    version="1.0",
    opnums=TRKSVR_OPNUMS,
)



### ms-dfsrh.idl
"""
RPC definitions for the following interfaces:
- IADProxy (v0.0): 4BB8AB1D-9EF9-4100-8EB6-DD4B4E418B72
- IADProxy2 (v0.0): C4B0C7D9-ABE0-4733-A1E1-9FDEDF260C7A
- IServerHealthReport (v0.0): E65E8028-83E8-491b-9AF7-AAF6BD51A0CE
- IServerHealthReport2 (v0.0): 20D15747-6C48-4254-A358-65039FD8C63C
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRRecursiveField,
    NDRRefEmbPointerField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    register_com_interface,
)


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_R4 = 4
    VT_R8 = 5
    VT_CY = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH = 9
    VT_ERROR = 10
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_UNKNOWN = 13
    VT_DECIMAL = 14
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_INT = 22
    VT_UINT = 23
    VT_VOID = 24
    VT_HRESULT = 25
    VT_PTR = 26
    VT_SAFEARRAY = 27
    VT_CARRAY = 28
    VT_USERDEFINED = 29
    VT_LPSTR = 30
    VT_LPWSTR = 31
    VT_RECORD = 36
    VT_INT_PTR = 37
    VT_UINT_PTR = 38
    VT_ARRAY = 8192
    VT_BYREF = 16384


class SF_TYPE(IntEnum):
    SF_ERROR = VARENUM.VT_ERROR
    SF_I1 = VARENUM.VT_I1
    SF_I2 = VARENUM.VT_I2
    SF_I4 = VARENUM.VT_I4
    SF_I8 = VARENUM.VT_I8
    SF_BSTR = VARENUM.VT_BSTR
    SF_UNKNOWN = VARENUM.VT_UNKNOWN
    SF_DISPATCH = VARENUM.VT_DISPATCH
    SF_VARIANT = VARENUM.VT_VARIANT
    SF_RECORD = VARENUM.VT_RECORD
    SF_HAVEIID = VARENUM.VT_UNKNOWN | 32768


class SAFEARR_BSTR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="aBstr"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "aBstr", [], FLAGGED_WORD_BLOB, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class MInterfacePointer(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["abData"]
    fields_desc = [
        NDRIntField("ulCntData", None, size_of="abData"),
        NDRConfStrLenField(
            "abData", "", size_is=lambda pkt: pkt.ulCntData, conformant_in_struct=True
        ),
    ]


class SAFEARR_UNKNOWN(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apUnknown"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apUnknown", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class SAFEARR_DISPATCH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apDispatch"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apDispatch", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class CURRENCY(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("int64", 0)]


class SAFEARRAYBOUND(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("cElements", 0), NDRSignedIntField("lLbound", 0)]


class SAFEARRAY(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["rgsabound"]
    fields_desc = [
        NDRShortField("cDims", None, size_of="rgsabound"),
        NDRShortField("fFeatures", 0),
        NDRIntField("cbElements", 0),
        NDRIntField("cLocks", 0),
        NDRRecursiveField("uArrayStructs"),
        NDRConfPacketListField(
            "rgsabound",
            [],
            SAFEARRAYBOUND,
            size_is=lambda pkt: pkt.cDims,
            conformant_in_struct=True,
        ),
    ]


class CreateObject_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("domainControllerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("distinguishedName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(NDRPacketField("attributes", SAFEARRAY(), SAFEARRAY)),
        NDRPacketField(
            "verifyNameDomainControllerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
    ]


class CreateObject_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteObject_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("domainControllerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("distinguishedName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class DeleteObject_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ModifyObject_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("domainControllerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("distinguishedName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(NDRPacketField("attributes", SAFEARRAY(), SAFEARRAY)),
    ]


class ModifyObject_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IADPROXY_OPNUMS = {
    0: DceRpcOp(CreateObject_Request, CreateObject_Response),
    1: DceRpcOp(DeleteObject_Request, DeleteObject_Response),
    2: DceRpcOp(ModifyObject_Request, ModifyObject_Response),
}
register_com_interface(
    name="IADProxy",
    uuid=uuid.UUID("4BB8AB1D-9EF9-4100-8EB6-DD4B4E418B72"),
    opnums=IADPROXY_OPNUMS,
)


class CreateObject_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("domainControllerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("distinguishedName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(NDRPacketField("attributes", SAFEARRAY(), SAFEARRAY)),
        NDRPacketField(
            "verifyNameDomainControllerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
        NDRPacketField(
            "networkNameResourceName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
    ]


class CreateObject_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteObject_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("domainControllerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("distinguishedName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField(
            "networkNameResourceName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
    ]


class DeleteObject_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ModifyObject_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("domainControllerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("distinguishedName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(NDRPacketField("attributes", SAFEARRAY(), SAFEARRAY)),
        NDRPacketField(
            "networkNameResourceName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
    ]


class ModifyObject_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IADPROXY2_OPNUMS = {
    0: DceRpcOp(CreateObject_Request, CreateObject_Response),
    1: DceRpcOp(DeleteObject_Request, DeleteObject_Response),
    2: DceRpcOp(ModifyObject_Request, ModifyObject_Response),
}
register_com_interface(
    name="IADProxy2",
    uuid=uuid.UUID("C4B0C7D9-ABE0-4733-A1E1-9FDEDF260C7A"),
    opnums=IADPROXY2_OPNUMS,
)


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class GetReport_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("replicationGroupGuid", GUID(), GUID),
        NDRPacketField("referenceMember", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(
            NDRPacketField("referenceVersionVectors", SAFEARRAY(), SAFEARRAY)
        ),
        NDRSignedIntField("flags", 0),
    ]


class GetReport_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("memberVersionVectors", SAFEARRAY(), SAFEARRAY)
        ),
        NDRFullPointerField(
            NDRPacketField("reportXML", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetCompressedReport_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("replicationGroupGuid", GUID(), GUID),
        NDRPacketField("referenceMember", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(
            NDRPacketField("referenceVersionVectors", SAFEARRAY(), SAFEARRAY)
        ),
        NDRSignedIntField("flags", 0),
    ]


class GetCompressedReport_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("memberVersionVectors", SAFEARRAY(), SAFEARRAY)
        ),
        NDRFullPointerField(
            NDRPacketField("reportCompressed", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRSignedIntField("uncompressedReportSize", 0),
        NDRIntField("status", 0),
    ]


class GetRawReportEx_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("replicationGroupGuid", GUID(), GUID),
        NDRFullPointerField(NDRPacketField("reportOptions", SAFEARRAY(), SAFEARRAY)),
    ]


class GetRawReportEx_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("report", SAFEARRAY(), SAFEARRAY)),
        NDRIntField("status", 0),
    ]


class GetReferenceVersionVectors_Request(NDRPacket):
    fields_desc = [NDRPacketField("replicationGroupGuid", GUID(), GUID)]


class GetReferenceVersionVectors_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("referenceVersionVectors", SAFEARRAY(), SAFEARRAY)
        ),
        NDRIntField("status", 0),
    ]


class GetReferenceBacklogCounts_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("flatMemberVersionVectors", SAFEARRAY(), SAFEARRAY)
        )
    ]


class GetReferenceBacklogCounts_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("backlogCounts", SAFEARRAY(), SAFEARRAY)),
        NDRIntField("status", 0),
    ]


ISERVERHEALTHREPORT_OPNUMS = {
    0: DceRpcOp(GetReport_Request, GetReport_Response),
    1: DceRpcOp(GetCompressedReport_Request, GetCompressedReport_Response),
    2: DceRpcOp(GetRawReportEx_Request, GetRawReportEx_Response),
    3: DceRpcOp(
        GetReferenceVersionVectors_Request, GetReferenceVersionVectors_Response
    ),
    # 4: Opnum7NotUsedOnWire,
    5: DceRpcOp(GetReferenceBacklogCounts_Request, GetReferenceBacklogCounts_Response),
}
register_com_interface(
    name="IServerHealthReport",
    uuid=uuid.UUID("E65E8028-83E8-491b-9AF7-AAF6BD51A0CE"),
    opnums=ISERVERHEALTHREPORT_OPNUMS,
)


class GetReport_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("replicationGroupGuid", GUID(), GUID),
        NDRPacketField("referenceMember", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("serverName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(
            NDRPacketField("referenceVersionVectors", SAFEARRAY(), SAFEARRAY)
        ),
        NDRSignedIntField("flags", 0),
    ]


class GetReport_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("memberVersionVectors", SAFEARRAY(), SAFEARRAY)
        ),
        NDRFullPointerField(
            NDRPacketField("reportXML", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetCompressedReport_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("replicationGroupGuid", GUID(), GUID),
        NDRPacketField("referenceMember", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("serverName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRFullPointerField(
            NDRPacketField("referenceVersionVectors", SAFEARRAY(), SAFEARRAY)
        ),
        NDRSignedIntField("flags", 0),
    ]


class GetCompressedReport_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("memberVersionVectors", SAFEARRAY(), SAFEARRAY)
        ),
        NDRFullPointerField(
            NDRPacketField("reportCompressed", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRSignedIntField("uncompressedReportSize", 0),
        NDRIntField("status", 0),
    ]


ISERVERHEALTHREPORT2_OPNUMS = {
    0: DceRpcOp(GetReport_Request, GetReport_Response),
    1: DceRpcOp(GetCompressedReport_Request, GetCompressedReport_Response),
}
register_com_interface(
    name="IServerHealthReport2",
    uuid=uuid.UUID("20D15747-6C48-4254-A358-65039FD8C63C"),
    opnums=ISERVERHEALTHREPORT2_OPNUMS,
)



### ms-even.idl
"""
RPC definitions for the following interfaces:
- eventlog (v0.0): 82273FDC-E32A-18C3-3F78-827929DC23EA
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedByteField,
    register_dcerpc_interface,
)


class PRPC_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class ElfrClearELFW_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRFullPointerField(
            NDRPacketField("BackupFileName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING)
        ),
    ]


class ElfrClearELFW_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ElfrBackupELFW_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("BackupFileName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
    ]


class ElfrBackupELFW_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ElfrCloseEL_Request(NDRPacket):
    fields_desc = [NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle)]


class ElfrCloseEL_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ElfrDeregisterEventSource_Request(NDRPacket):
    fields_desc = [NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle)]


class ElfrDeregisterEventSource_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ElfrNumberOfRecords_Request(NDRPacket):
    fields_desc = [NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle)]


class ElfrNumberOfRecords_Response(NDRPacket):
    fields_desc = [NDRIntField("NumberOfRecords", 0), NDRIntField("status", 0)]


class ElfrOldestRecord_Request(NDRPacket):
    fields_desc = [NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle)]


class ElfrOldestRecord_Response(NDRPacket):
    fields_desc = [NDRIntField("OldestRecordNumber", 0), NDRIntField("status", 0)]


class RPC_CLIENT_ID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("UniqueProcess", 0), NDRIntField("UniqueThread", 0)]


class ElfrChangeNotify_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("ClientId", RPC_CLIENT_ID(), RPC_CLIENT_ID),
        NDRIntField("Event", 0),
    ]


class ElfrChangeNotify_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ElfrOpenELW_Request(NDRPacket):
    fields_desc = [
        NDRShortField("UNCServerName", 0),
        NDRPacketField("ModuleName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
        NDRPacketField("RegModuleName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
        NDRIntField("MajorVersion", 0),
        NDRIntField("MinorVersion", 0),
    ]


class ElfrOpenELW_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ElfrRegisterEventSourceW_Request(NDRPacket):
    fields_desc = [
        NDRShortField("UNCServerName", 0),
        NDRPacketField("ModuleName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
        NDRPacketField("RegModuleName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
        NDRIntField("MajorVersion", 0),
        NDRIntField("MinorVersion", 0),
    ]


class ElfrRegisterEventSourceW_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ElfrOpenBELW_Request(NDRPacket):
    fields_desc = [
        NDRShortField("UNCServerName", 0),
        NDRPacketField("BackupFileName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
        NDRIntField("MajorVersion", 0),
        NDRIntField("MinorVersion", 0),
    ]


class ElfrOpenBELW_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ElfrReadELW_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("ReadFlags", 0),
        NDRIntField("RecordOffset", 0),
        NDRIntField("NumberOfBytesToRead", 0),
    ]


class ElfrReadELW_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("Buffer", "", size_is=lambda pkt: pkt.NumberOfBytesToRead),
        NDRIntField("NumberOfBytesRead", 0),
        NDRIntField("MinNumberOfBytesNeeded", 0),
        NDRIntField("status", 0),
    ]


class RPC_SID_IDENTIFIER_AUTHORITY(NDRPacket):
    fields_desc = [StrFixedLenField("Value", "", length=6)]


class PRPC_SID(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["SubAuthority"]
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("SubAuthorityCount", None, size_of="SubAuthority"),
        NDRPacketField(
            "IdentifierAuthority",
            RPC_SID_IDENTIFIER_AUTHORITY(),
            RPC_SID_IDENTIFIER_AUTHORITY,
        ),
        NDRConfFieldListField(
            "SubAuthority",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.SubAuthorityCount,
            conformant_in_struct=True,
        ),
    ]


class ElfrReportEventW_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Time", 0),
        NDRShortField("EventType", 0),
        NDRShortField("EventCategory", 0),
        NDRIntField("EventID", 0),
        NDRShortField("NumStrings", None, size_of="Strings"),
        NDRIntField("DataSize", None, size_of="Data"),
        NDRPacketField("ComputerName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
        NDRFullPointerField(NDRPacketField("UserSID", PRPC_SID(), PRPC_SID)),
        NDRConfPacketListField(
            "Strings",
            [],
            PRPC_UNICODE_STRING,
            size_is=lambda pkt: pkt.NumStrings,
            ptr_pack=True,
        ),
        NDRConfStrLenField("Data", "", size_is=lambda pkt: pkt.DataSize),
        NDRShortField("Flags", 0),
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
        NDRFullPointerField(NDRIntField("TimeWritten", 0)),
    ]


class ElfrReportEventW_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
        NDRFullPointerField(NDRIntField("TimeWritten", 0)),
        NDRIntField("status", 0),
    ]


class PRPC_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", 0),
        NDRShortField("MaximumLength", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfStrLenField("Buffer", "", size_is=lambda pkt: pkt.MaximumLength),
            deferred=True,
        ),
    ]


class ElfrClearELFA_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRFullPointerField(
            NDRPacketField("BackupFileName", PRPC_STRING(), PRPC_STRING)
        ),
    ]


class ElfrClearELFA_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ElfrBackupELFA_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("BackupFileName", PRPC_STRING(), PRPC_STRING),
    ]


class ElfrBackupELFA_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ElfrOpenELA_Request(NDRPacket):
    fields_desc = [
        NDRSignedByteField("UNCServerName", 0),
        NDRPacketField("ModuleName", PRPC_STRING(), PRPC_STRING),
        NDRPacketField("RegModuleName", PRPC_STRING(), PRPC_STRING),
        NDRIntField("MajorVersion", 0),
        NDRIntField("MinorVersion", 0),
    ]


class ElfrOpenELA_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ElfrRegisterEventSourceA_Request(NDRPacket):
    fields_desc = [
        NDRSignedByteField("UNCServerName", 0),
        NDRPacketField("ModuleName", PRPC_STRING(), PRPC_STRING),
        NDRPacketField("RegModuleName", PRPC_STRING(), PRPC_STRING),
        NDRIntField("MajorVersion", 0),
        NDRIntField("MinorVersion", 0),
    ]


class ElfrRegisterEventSourceA_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ElfrOpenBELA_Request(NDRPacket):
    fields_desc = [
        NDRSignedByteField("UNCServerName", 0),
        NDRPacketField("BackupFileName", PRPC_STRING(), PRPC_STRING),
        NDRIntField("MajorVersion", 0),
        NDRIntField("MinorVersion", 0),
    ]


class ElfrOpenBELA_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class ElfrReadELA_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("ReadFlags", 0),
        NDRIntField("RecordOffset", 0),
        NDRIntField("NumberOfBytesToRead", 0),
    ]


class ElfrReadELA_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("Buffer", "", size_is=lambda pkt: pkt.NumberOfBytesToRead),
        NDRIntField("NumberOfBytesRead", 0),
        NDRIntField("MinNumberOfBytesNeeded", 0),
        NDRIntField("status", 0),
    ]


class ElfrReportEventA_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Time", 0),
        NDRShortField("EventType", 0),
        NDRShortField("EventCategory", 0),
        NDRIntField("EventID", 0),
        NDRShortField("NumStrings", None, size_of="Strings"),
        NDRIntField("DataSize", None, size_of="Data"),
        NDRPacketField("ComputerName", PRPC_STRING(), PRPC_STRING),
        NDRFullPointerField(NDRPacketField("UserSID", PRPC_SID(), PRPC_SID)),
        NDRConfPacketListField(
            "Strings",
            [],
            PRPC_STRING,
            size_is=lambda pkt: pkt.NumStrings,
            ptr_pack=True,
        ),
        NDRConfStrLenField("Data", "", size_is=lambda pkt: pkt.DataSize),
        NDRShortField("Flags", 0),
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
        NDRFullPointerField(NDRIntField("TimeWritten", 0)),
    ]


class ElfrReportEventA_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
        NDRFullPointerField(NDRIntField("TimeWritten", 0)),
        NDRIntField("status", 0),
    ]


class ElfrGetLogInformation_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("InfoLevel", 0),
        NDRIntField("cbBufSize", 0),
    ]


class ElfrGetLogInformation_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("lpBuffer", "", size_is=lambda pkt: pkt.cbBufSize),
        NDRIntField("pcbBytesNeeded", 0),
        NDRIntField("status", 0),
    ]


class ElfrReportEventAndSourceW_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Time", 0),
        NDRShortField("EventType", 0),
        NDRShortField("EventCategory", 0),
        NDRIntField("EventID", 0),
        NDRPacketField("SourceName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
        NDRShortField("NumStrings", None, size_of="Strings"),
        NDRIntField("DataSize", None, size_of="Data"),
        NDRPacketField("ComputerName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
        NDRFullPointerField(NDRPacketField("UserSID", PRPC_SID(), PRPC_SID)),
        NDRConfPacketListField(
            "Strings",
            [],
            PRPC_UNICODE_STRING,
            size_is=lambda pkt: pkt.NumStrings,
            ptr_pack=True,
        ),
        NDRConfStrLenField("Data", "", size_is=lambda pkt: pkt.DataSize),
        NDRShortField("Flags", 0),
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
        NDRFullPointerField(NDRIntField("TimeWritten", 0)),
    ]


class ElfrReportEventAndSourceW_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
        NDRFullPointerField(NDRIntField("TimeWritten", 0)),
        NDRIntField("status", 0),
    ]


class PFILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class ElfrReportEventExW_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("TimeGenerated", PFILETIME(), PFILETIME),
        NDRShortField("EventType", 0),
        NDRShortField("EventCategory", 0),
        NDRIntField("EventID", 0),
        NDRShortField("NumStrings", None, size_of="Strings"),
        NDRIntField("DataSize", None, size_of="Data"),
        NDRPacketField("ComputerName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING),
        NDRFullPointerField(NDRPacketField("UserSID", PRPC_SID(), PRPC_SID)),
        NDRConfPacketListField(
            "Strings",
            [],
            PRPC_UNICODE_STRING,
            size_is=lambda pkt: pkt.NumStrings,
            ptr_pack=True,
        ),
        NDRConfStrLenField("Data", "", size_is=lambda pkt: pkt.DataSize),
        NDRShortField("Flags", 0),
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
    ]


class ElfrReportEventExW_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
        NDRIntField("status", 0),
    ]


class ElfrReportEventExA_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("LogHandle", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("TimeGenerated", PFILETIME(), PFILETIME),
        NDRShortField("EventType", 0),
        NDRShortField("EventCategory", 0),
        NDRIntField("EventID", 0),
        NDRShortField("NumStrings", None, size_of="Strings"),
        NDRIntField("DataSize", None, size_of="Data"),
        NDRPacketField("ComputerName", PRPC_STRING(), PRPC_STRING),
        NDRFullPointerField(NDRPacketField("UserSID", PRPC_SID(), PRPC_SID)),
        NDRConfPacketListField(
            "Strings",
            [],
            PRPC_STRING,
            size_is=lambda pkt: pkt.NumStrings,
            ptr_pack=True,
        ),
        NDRConfStrLenField("Data", "", size_is=lambda pkt: pkt.DataSize),
        NDRShortField("Flags", 0),
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
    ]


class ElfrReportEventExA_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRIntField("RecordNumber", 0)),
        NDRIntField("status", 0),
    ]


EVENTLOG_OPNUMS = {
    0: DceRpcOp(ElfrClearELFW_Request, ElfrClearELFW_Response),
    1: DceRpcOp(ElfrBackupELFW_Request, ElfrBackupELFW_Response),
    2: DceRpcOp(ElfrCloseEL_Request, ElfrCloseEL_Response),
    3: DceRpcOp(ElfrDeregisterEventSource_Request, ElfrDeregisterEventSource_Response),
    4: DceRpcOp(ElfrNumberOfRecords_Request, ElfrNumberOfRecords_Response),
    5: DceRpcOp(ElfrOldestRecord_Request, ElfrOldestRecord_Response),
    6: DceRpcOp(ElfrChangeNotify_Request, ElfrChangeNotify_Response),
    7: DceRpcOp(ElfrOpenELW_Request, ElfrOpenELW_Response),
    8: DceRpcOp(ElfrRegisterEventSourceW_Request, ElfrRegisterEventSourceW_Response),
    9: DceRpcOp(ElfrOpenBELW_Request, ElfrOpenBELW_Response),
    10: DceRpcOp(ElfrReadELW_Request, ElfrReadELW_Response),
    11: DceRpcOp(ElfrReportEventW_Request, ElfrReportEventW_Response),
    12: DceRpcOp(ElfrClearELFA_Request, ElfrClearELFA_Response),
    13: DceRpcOp(ElfrBackupELFA_Request, ElfrBackupELFA_Response),
    14: DceRpcOp(ElfrOpenELA_Request, ElfrOpenELA_Response),
    15: DceRpcOp(ElfrRegisterEventSourceA_Request, ElfrRegisterEventSourceA_Response),
    16: DceRpcOp(ElfrOpenBELA_Request, ElfrOpenBELA_Response),
    17: DceRpcOp(ElfrReadELA_Request, ElfrReadELA_Response),
    18: DceRpcOp(ElfrReportEventA_Request, ElfrReportEventA_Response),
    # 19: Opnum19NotUsedOnWire,
    # 20: Opnum20NotUsedOnWire,
    # 21: Opnum21NotUsedOnWire,
    22: DceRpcOp(ElfrGetLogInformation_Request, ElfrGetLogInformation_Response),
    # 23: Opnum23NotUsedOnWire,
    24: DceRpcOp(ElfrReportEventAndSourceW_Request, ElfrReportEventAndSourceW_Response),
    25: DceRpcOp(ElfrReportEventExW_Request, ElfrReportEventExW_Response),
    26: DceRpcOp(ElfrReportEventExA_Request, ElfrReportEventExA_Response),
}
register_dcerpc_interface(
    name="eventlog",
    uuid=uuid.UUID("82273FDC-E32A-18C3-3F78-827929DC23EA"),
    version="0.0",
    opnums=EVENTLOG_OPNUMS,
)



### ms-tsrap.idl
"""
RPC definitions for the following interfaces:
- IManageTelnetSessions (v0.0): 034634FD-BA3F-11D1-856A-00A0C944138C
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    register_com_interface,
)


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class GetTelnetSessions_Request(NDRPacket):
    fields_desc = []


class GetTelnetSessions_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pszSessionData", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class TerminateSession_Request(NDRPacket):
    fields_desc = [NDRIntField("dwUniqueId", 0)]


class TerminateSession_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SendMsgToASession_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwUniqueId", 0),
        NDRPacketField("szMsg", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class SendMsgToASession_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IMANAGETELNETSESSIONS_OPNUMS = {
    0: DceRpcOp(GetTelnetSessions_Request, GetTelnetSessions_Response),
    1: DceRpcOp(TerminateSession_Request, TerminateSession_Response),
    2: DceRpcOp(SendMsgToASession_Request, SendMsgToASession_Response),
}
register_com_interface(
    name="IManageTelnetSessions",
    uuid=uuid.UUID("034634FD-BA3F-11D1-856A-00A0C944138C"),
    opnums=IMANAGETELNETSESSIONS_OPNUMS,
)
